"""Mostly a compatibility module"""

from riakasaurus import riak_object, mapreduce, bucket, client

RiakClient = client.RiakClient
RiakBucket = bucket.RiakBucket
RiakObject = riak_object.RiakObject
RiakMapReduce = mapreduce.RiakMapReduce
RiakLink = mapreduce.RiakLink
